+{
   locale_version => 1.14,
   entry => <<'ENTRY', # for DUCET v7.0.0
010D      ; [.193F.0020.0002] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.193F.0020.0002] # LATIN SMALL LETTER C WITH CARON
010C      ; [.193F.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.193F.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0107      ; [.1940.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0063 0301 ; [.1940.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0063 0341 ; [.1940.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0106      ; [.1940.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
0043 0301 ; [.1940.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
0043 0341 ; [.1940.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
0064 017E ; [.1954.0020.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER Z WITH CARON>
01C6      ; [.1954.0020.0003] # LATIN SMALL LETTER DZ WITH CARON
0044 017E ; [.1954.0020.0007][.0000.0000.0001] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z WITH CARON>
01C5      ; [.1954.0020.0007][.0000.0000.0007] # LATIN CAPITAL LETTER D WITH SMALL LETTER Z WITH CARON
0044 017D ; [.1954.0020.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z WITH CARON>
01C4      ; [.1954.0020.0009] # LATIN CAPITAL LETTER DZ WITH CARON
0064 007A 030C ; [.1954.0020.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER Z, COMBINING CARON>
0044 007A 030C ; [.1954.0020.0007][.0000.0000.0001] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z, COMBINING CARON>
0044 005A 030C ; [.1954.0020.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z, COMBINING CARON>
0111      ; [.1955.0020.0002] # LATIN SMALL LETTER D WITH STROKE
0064 0335 ; [.1955.0020.0002] # LATIN SMALL LETTER D WITH STROKE
0110      ; [.1955.0020.0008] # LATIN CAPITAL LETTER D WITH STROKE
0044 0335 ; [.1955.0020.0008] # LATIN CAPITAL LETTER D WITH STROKE
006C 006A ; [.1A3C.0020.0002] # <LATIN SMALL LETTER L, LATIN SMALL LETTER J>
01C9      ; [.1A3C.0020.0003] # LATIN SMALL LETTER LJ
004C 006A ; [.1A3C.0020.0007][.0000.0000.0001] # <LATIN CAPITAL LETTER L, LATIN SMALL LETTER J>
01C8      ; [.1A3C.0020.0007][.0000.0000.0007] # LATIN CAPITAL LETTER L WITH SMALL LETTER J
004C 004A ; [.1A3C.0020.0008] # <LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER J>
01C7      ; [.1A3C.0020.0009] # LATIN CAPITAL LETTER LJ
006E 006A ; [.1A7E.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER J>
01CC      ; [.1A7E.0020.0003] # LATIN SMALL LETTER NJ
004E 006A ; [.1A7E.0020.0007][.0000.0000.0001] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER J>
01CB      ; [.1A7E.0020.0007][.0000.0000.0007] # LATIN CAPITAL LETTER N WITH SMALL LETTER J
004E 004A ; [.1A7E.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER J>
01CA      ; [.1A7E.0020.0009] # LATIN CAPITAL LETTER NJ
0161      ; [.1B34.0020.0002] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.1B34.0020.0002] # LATIN SMALL LETTER S WITH CARON
0160      ; [.1B34.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.1B34.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
017E      ; [.1BE4.0020.0002] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.1BE4.0020.0002] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.1BE4.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.1BE4.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
ENTRY
};
